@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013 Canonical, Ltd.
+ * Copyright (C) 2014 Canonical, Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 3, as published
@@ -7,7 +7,7 @@
  *
  * This library is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
- * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+ * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
  * details.
  *
  * You should have received a copy of the GNU General Public License
@@ -21,11 +21,6 @@
 
 struct ActionModelPrivate;
 
-enum ActionsRoles {
-    RoleActionLabel = Qt::UserRole + 1,
-    RoleActionId    = Qt::UserRole + 2
-};
-
 class ActionModel : public QStringListModel {
 	Q_OBJECT
 
@@ -37,7 +32,14 @@ public:
     virtual QVariant data(const QModelIndex &index, int role) const;
     virtual QHash<int, QByteArray> roleNames() const;
 
-    void insertAction(QString id, QString label);
+    Q_ENUMS(ActionsRoles)
+    enum ActionsRoles {
+        RoleActionLabel = Qt::UserRole + 1,
+        RoleActionId    = Qt::UserRole + 2
+    };
+    Q_INVOKABLE QVariant data(int row, int role) const;
+
+    void insertAction(const QString &id, const QString &label);
 
 private:
     QScopedPointer<ActionModelPrivate> p;
