@@ -21,6 +21,7 @@
 #include "NotificationServer.h"
 #include "Notification.h"
 #include <string>
+#include <QXmlStreamReader>
 
 using namespace std;
 
@@ -54,7 +55,7 @@ Notification::Notification(QObject *parent) : QObject(parent), p(new Notificatio
     p->actionsModel = new ActionModel();
 }
 
-Notification::Notification(NotificationID id, int displayTime, const Urgency ur, QString text, Type type, NotificationServer *srv, QObject *parent) :
+Notification::Notification(NotificationID id, int displayTime, const Urgency ur, const QString &text, Type type, NotificationServer *srv, QObject *parent) :
                 QObject(parent), p(new NotificationPrivate()) {
     p->id = id;
     p->urg = ur;
@@ -67,7 +68,7 @@ Notification::Notification(NotificationID id, int displayTime, const Urgency ur,
 }
 
 Notification::Notification(NotificationID id, int displayTime, const Urgency ur, Type type, NotificationServer *srv, QObject *parent) :
-    Notification(id, displayTime, ur, "", type, srv, parent){
+    Notification(id, displayTime, ur, QString(), type, srv, parent){
     p->actionsModel = new ActionModel();
 }
 
@@ -80,10 +81,11 @@ QString Notification::getBody() const {
     return p->body;
 }
 
-void Notification::setBody(const QString text) {
-    if(p->body != text) {
-        p->body = text;
-        Q_EMIT bodyChanged(text);
+void Notification::setBody(const QString &text) {
+    QString filtered = filterText(text);
+    if(p->body != filtered) {
+        p->body = filtered;
+        Q_EMIT bodyChanged(p->body);
         Q_EMIT dataChanged(p->id);
     }
 }
@@ -112,7 +114,7 @@ QString Notification::getIcon() const {
     return p->icon;
 }
 
-void Notification::setIcon(QString icon) {
+void Notification::setIcon(const QString &icon) {
     if (icon.startsWith(" ") || icon.size() == 0) {
         p->icon = nullptr;
     }
@@ -132,8 +134,18 @@ QString Notification::getSecondaryIcon() const {
     return p->secondaryIcon;
 }
 
-void Notification::setSecondaryIcon(QString secondaryIcon) {
-    p->secondaryIcon = secondaryIcon;
+void Notification::setSecondaryIcon(const QString &secondaryIcon) {
+    if (secondaryIcon.startsWith(" ") || secondaryIcon.size() == 0) {
+        p->secondaryIcon = nullptr;
+    }
+    else {
+        p->secondaryIcon = secondaryIcon;
+
+        if (secondaryIcon.indexOf("/") == -1) {
+            p->secondaryIcon.prepend("image://theme/");
+        }
+    }
+
     Q_EMIT secondaryIconChanged(p->secondaryIcon);
     Q_EMIT dataChanged(p->id);
 }
@@ -142,9 +154,10 @@ QString Notification::getSummary() const {
     return p->summary;
 }
 
-void Notification::setSummary(QString summary) {
-    if(p->summary != summary) {
-        p->summary = summary;
+void Notification::setSummary(const QString &summary) {
+    QString filtered = filterText(summary);
+    if(p->summary != filtered) {
+        p->summary = filtered;
         Q_EMIT summaryChanged(p->summary);
         Q_EMIT dataChanged(p->id);
     }
@@ -181,7 +194,7 @@ ActionModel* Notification::getActions() const {
     return p->actionsModel;
 }
 
-void Notification::setActions(QStringList actions) {
+void Notification::setActions(const QStringList &actions) {
     if(p->actions != actions) {
         p->actions = actions;
         Q_EMIT actionsChanged(p->actions);
@@ -192,6 +205,19 @@ void Notification::setActions(QStringList actions) {
     }
 }
 
+void Notification::detachFromServer() {
+    /*
+     * FIXME. This function should not exist. In fact
+     * a notification should not need to know the server
+     * it is connected to. Instead the way p->server
+     * is used needs to be converted into a proper
+     * Qt signal. Unfortunately, due to deadlines,
+     * we did not have time. The next time this is worked
+     * on, fix this properly.
+     */
+    p->server = nullptr;
+}
+
 QVariantMap Notification::getHints() const {
     return p->hints;
 }
@@ -211,7 +237,7 @@ void Notification::onDisplayed() {
 
 }
 
-void Notification::invokeAction(const QString action) {
+void Notification::invokeAction(const QString &action) {
     for(int i=0; i<p->actions.size(); i++) {
         if(p->actions[i] == action) {
             p->server->invokeAction(p->id, action);
@@ -221,3 +247,25 @@ void Notification::invokeAction(const QString action) {
     }
     fprintf(stderr, "Error: tried to invoke action not in actionList.\n");
 }
+
+void Notification::close() {
+    Q_EMIT completed(p->id);
+}
+
+QString Notification::filterText(const QString& text) {
+    QString plaintext;
+
+    QXmlStreamReader reader("<p>" + text + "</p>");
+
+    while (!reader.atEnd() && !reader.hasError()) {
+        QXmlStreamReader::TokenType token = reader.readNext();
+        if (token == QXmlStreamReader::Characters) {
+            plaintext.append(reader.text().toString());
+        }
+    }
+    if (reader.hasError()) {
+        // Not valid xml. Return as is.
+        return text;
+    }
+    return plaintext;
+}
