@@ -19,6 +19,9 @@
 
 #include "NotificationModel.h"
 #include "Notification.h"
+
+#include <unity/shell/notifications/ModelInterface.h>
+
 #include <QTimer>
 #include <QList>
 #include <QVector>
@@ -26,6 +29,8 @@
 #include <QStringListModel>
 #include <QQmlEngine>
 
+using namespace unity::shell::notifications;
+
 struct NotificationModelPrivate {
     QList<QSharedPointer<Notification> > displayedNotifications;
     QTimer timer;
@@ -40,6 +45,7 @@ bool notificationCompare(const QSharedPointer<Notification> &first, const QShare
 }
 
 NotificationModel::NotificationModel(QObject *parent) : QAbstractListModel(parent), p(new NotificationModelPrivate) {
+    p->displayedNotifications.append(QSharedPointer<Notification>(new Notification(0, -1, Notification::Normal, QString(), Notification::PlaceHolder)));
     connect(&(p->timer), SIGNAL(timeout()), this, SLOT(timeout()));
     p->timer.setSingleShot(true);
 }
@@ -60,7 +66,7 @@ NotificationModel::~NotificationModel() {
 }
 
 int NotificationModel::rowCount(const QModelIndex &parent) const {
-    //printf("Count %d\n", displayedNotifications.size());
+    //printf("Count %d\n", p->displayedNotifications.size());
     return p->displayedNotifications.size();
 }
 
@@ -70,37 +76,37 @@ QVariant NotificationModel::data(const QModelIndex &index, int role) const {
             return QVariant();
 
     switch(role) {
-        case RoleType:
+        case ModelInterface::RoleType:
             return QVariant(p->displayedNotifications[index.row()]->getType());
 
-        case RoleUrgency:
+        case ModelInterface::RoleUrgency:
             return QVariant(p->displayedNotifications[index.row()]->getUrgency());
 
-        case RoleId:
+        case ModelInterface::RoleId:
             return QVariant(p->displayedNotifications[index.row()]->getID());
 
-        case RoleSummary:
+        case ModelInterface::RoleSummary:
             return QVariant(p->displayedNotifications[index.row()]->getSummary());
 
-        case RoleBody:
+        case ModelInterface::RoleBody:
             return QVariant(p->displayedNotifications[index.row()]->getBody());
 
-        case RoleValue:
+        case ModelInterface::RoleValue:
             return QVariant(p->displayedNotifications[index.row()]->getValue());
 
-        case RoleIcon:
+        case ModelInterface::RoleIcon:
             return QVariant(p->displayedNotifications[index.row()]->getIcon());
 
-        case RoleSecondaryIcon:
+        case ModelInterface::RoleSecondaryIcon:
             return QVariant(p->displayedNotifications[index.row()]->getSecondaryIcon());
 
-        case RoleActions:
+        case ModelInterface::RoleActions:
             return QVariant::fromValue(p->displayedNotifications[index.row()]->getActions());
 
-        case RoleHints:
+        case ModelInterface::RoleHints:
             return QVariant(p->displayedNotifications[index.row()]->getHints());
 
-        case RoleNotification:
+        case ModelInterface::RoleNotification:
             return QVariant(p->displayedNotifications[index.row()]);
 
         default:
@@ -108,7 +114,7 @@ QVariant NotificationModel::data(const QModelIndex &index, int role) const {
     }
 }
 
-void NotificationModel::insertNotification(QSharedPointer<Notification> n) {
+void NotificationModel::insertNotification(const QSharedPointer<Notification> &n) {
     if(numNotifications() >= maxNotifications)
         return; // Just ignore. Maybe we should throw()?
     int remaining = p->timer.remainingTime();
@@ -156,7 +162,7 @@ QSharedPointer<Notification> NotificationModel::getNotification(NotificationID i
     return empty;
 }
 
-QSharedPointer<Notification> NotificationModel::getNotification(QString summary) const {
+QSharedPointer<Notification> NotificationModel::getNotification(const QString &summary) const {
     for(int i=0; i<p->ephemeralQueue.size(); i++) {
         if(p->ephemeralQueue[i]->getSummary() == summary) {
             return p->ephemeralQueue[i];
@@ -182,6 +188,15 @@ QSharedPointer<Notification> NotificationModel::getNotification(QString summary)
     return empty;
 }
 
+QSharedPointer<Notification> NotificationModel::getDisplayedNotification(int index) const {
+    if (index < p->displayedNotifications.size()) {
+        return p->displayedNotifications[index];
+    } else {
+        QSharedPointer<Notification> empty;
+        return empty;
+    }
+}
+
 bool NotificationModel::hasNotification(NotificationID id) const {
     return !(getNotification(id).isNull());
 }
@@ -294,7 +309,8 @@ void NotificationModel::pruneExpired() {
     for(int i=p->displayedNotifications.size()-1; i>=0; i--) {
         QSharedPointer<Notification> n = p->displayedNotifications[i];
         int shownTime = p->displayTimes[n->getID()];
-        if(shownTime >= n->getDisplayTime()) {
+        int totalTime = n->getDisplayTime();
+        if(totalTime >= 0 && shownTime >= totalTime) {
             deleteFromVisible(i);
         }
     }
@@ -323,17 +339,19 @@ int NotificationModel::nextTimeout() const {
     for(int i=0; i<p->displayedNotifications.size(); i++) {
         QSharedPointer<Notification> n = p->displayedNotifications[i];
         int totalTime = n->getDisplayTime();
-        int shownTime = p->displayTimes[n->getID()];
-        int remainingTime = totalTime - shownTime;
-        if(remainingTime < 0)
-            remainingTime = 0;
-        if(remainingTime < mintime)
-            mintime = remainingTime;
+        if (totalTime >= 0) {
+            int shownTime = p->displayTimes[n->getID()];
+            int remainingTime = totalTime - shownTime;
+            if(remainingTime < 0)
+                remainingTime = 0;
+            if(remainingTime < mintime)
+                mintime = remainingTime;
+        }
     }
     return mintime;
 }
 
-void NotificationModel::insertEphemeral(QSharedPointer<Notification> n) {
+void NotificationModel::insertEphemeral(const QSharedPointer<Notification> &n) {
     Q_ASSERT(n->getType() == Notification::Type::Ephemeral);
 
     if(showingNotificationOfType(Notification::Type::SnapDecision)) {
@@ -357,7 +375,7 @@ void NotificationModel::insertEphemeral(QSharedPointer<Notification> n) {
     }
 }
 
-void NotificationModel::insertInteractive(QSharedPointer<Notification> n) {
+void NotificationModel::insertInteractive(const QSharedPointer<Notification> &n) {
     Q_ASSERT(n->getType() == Notification::Type::Interactive);
 
     if(showingNotificationOfType(Notification::Type::SnapDecision)) {
@@ -384,7 +402,7 @@ void NotificationModel::insertInteractive(QSharedPointer<Notification> n) {
 }
 
 
-void NotificationModel::insertConfirmation(QSharedPointer<Notification> n) {
+void NotificationModel::insertConfirmation(const QSharedPointer<Notification> &n) {
     Q_ASSERT(n->getType() == Notification::Type::Confirmation);
     if(showingNotificationOfType(Notification::Type::Confirmation)) {
         deleteFirst(); // Synchronous is always first.
@@ -393,7 +411,7 @@ void NotificationModel::insertConfirmation(QSharedPointer<Notification> n) {
 }
 
 
-void NotificationModel::insertSnap(QSharedPointer<Notification> n) {
+void NotificationModel::insertSnap(const QSharedPointer<Notification> &n) {
     Q_ASSERT(n->getType() == Notification::Type::SnapDecision);
     removeNonSnap();
     int showing = countShowing(n->getType());
@@ -401,28 +419,39 @@ void NotificationModel::insertSnap(QSharedPointer<Notification> n) {
         int loc = findFirst(Notification::Type::SnapDecision);
         bool replaced = false;
         for(int i=0; i<showing; i++) {
-            if(p->displayedNotifications[loc+i]->getUrgency() < n->getUrgency()) {
+            if(p->displayedNotifications[loc+i]->getUrgency() > n->getUrgency()) {
                 QSharedPointer<Notification> lastShowing = p->displayedNotifications[loc+showing-1];
                 deleteFromVisible(loc+showing-1);
-                insertToVisible(n, loc+i);
+                insertToVisible(n, loc+i+1);
                 p->snapQueue.push_front(lastShowing);
                 replaced = true;
                 break;
             }
         }
+
         if(!replaced) {
             p->snapQueue.push_back(n);
         }
         qStableSort(p->snapQueue.begin(), p->snapQueue.end(), notificationCompare);
         Q_EMIT queueSizeChanged(queued());
     } else {
-        int loc = insertionPoint(n);
-        insertToVisible(n, loc);
+        bool inserted = false;
+        int loc = findFirst(Notification::Type::SnapDecision);
+        for(int i=0; i<showing; i++) {
+            if(p->displayedNotifications[loc+i]->getUrgency() > n->getUrgency()) {
+                insertToVisible(n, loc+i+1);
+                inserted = true;
+                break;
+            }
+        }
+
+        if (!inserted) {
+            insertToVisible(n, 1);
+        }
     }
 }
 
-int NotificationModel::insertionPoint(const QSharedPointer<Notification> n) const {
-    int i=0;
+int NotificationModel::insertionPoint(const QSharedPointer<Notification> &n) const {
     if(n->getType() == Notification::Type::SnapDecision) {
         int loc = findFirst(Notification::Type::SnapDecision);
         int numSnaps = countShowing(Notification::Type::SnapDecision);
@@ -433,6 +462,7 @@ int NotificationModel::insertionPoint(const QSharedPointer<Notification> n) cons
         }
         return loc+numSnaps;
     } else {
+        int i=0;
         for(; i<p->displayedNotifications.size(); i++) {
             if(p->displayedNotifications[i]->getType() > n->getType()) {
                 break;
@@ -442,7 +472,7 @@ int NotificationModel::insertionPoint(const QSharedPointer<Notification> n) cons
     }
 }
 
-void NotificationModel::insertToVisible(QSharedPointer<Notification> n, int location) {
+void NotificationModel::insertToVisible(const QSharedPointer<Notification> &n, int location) {
     if(location < 0)
         location = p->displayedNotifications.size();
     if(location > p->displayedNotifications.size()) {
@@ -452,7 +482,7 @@ void NotificationModel::insertToVisible(QSharedPointer<Notification> n, int loca
     //QModelIndex insertionPoint = QAbstractItemModel::createIndex(location, 0);
     //beginInsertRows(insertionPoint, location, location);
     QModelIndex insertionPoint = QModelIndex();
-    beginInsertRows(insertionPoint, rowCount(insertionPoint), rowCount(insertionPoint));
+    beginInsertRows(insertionPoint, location, location);
     p->displayedNotifications.insert(location, n);
     endInsertRows();
     p->displayTimes[n->getID()] = 0;
@@ -518,17 +548,17 @@ int NotificationModel::findFirst(const Notification::Type type) const {
 QHash<int, QByteArray> NotificationModel::roleNames() const {
     QHash<int, QByteArray> roles;
 
-    roles.insert(RoleType, "type");
-    roles.insert(RoleUrgency, "urgency");
-    roles.insert(RoleId, "id");
-    roles.insert(RoleSummary, "summary");
-    roles.insert(RoleBody, "body");
-    roles.insert(RoleValue, "value");
-    roles.insert(RoleIcon, "icon");
-    roles.insert(RoleSecondaryIcon, "secondaryIcon");
-    roles.insert(RoleActions, "actions");
-    roles.insert(RoleHints, "hints");
-    roles.insert(RoleNotification, "notification");
+    roles.insert(ModelInterface::RoleType, "type");
+    roles.insert(ModelInterface::RoleUrgency, "urgency");
+    roles.insert(ModelInterface::RoleId, "id");
+    roles.insert(ModelInterface::RoleSummary, "summary");
+    roles.insert(ModelInterface::RoleBody, "body");
+    roles.insert(ModelInterface::RoleValue, "value");
+    roles.insert(ModelInterface::RoleIcon, "icon");
+    roles.insert(ModelInterface::RoleSecondaryIcon, "secondaryIcon");
+    roles.insert(ModelInterface::RoleActions, "actions");
+    roles.insert(ModelInterface::RoleHints, "hints");
+    roles.insert(ModelInterface::RoleNotification, "notification");
 
     return roles;
 }
@@ -546,8 +576,10 @@ void NotificationModel::notificationUpdated(const NotificationID id) {
 }
 
 void NotificationModel::onDataChanged(unsigned int id) {
-    // FIXME: not really the right way to do it by assuming
-    // it's always the first notification being displayed
-    // that's affected by a potential data-change
-    Q_EMIT dataChanged(index(0, 0), index(0, 0));
+    for(int i=0; i<p->displayedNotifications.size(); i++) {
+        if(p->displayedNotifications[i]->getID() == id) {
+            Q_EMIT dataChanged(index(i, 0), index(i, 0));
+            break;
+        }
+    }
 }
