@@ -21,6 +21,7 @@
 #include "NotificationServer.h"
 #include "Notification.h"
 #include <QDBusMetaType>
+#include <QDebug>
 #include <QSharedPointer>
 
 NotificationServer::NotificationServer(NotificationModel &m, QObject *parent) :
@@ -38,12 +39,6 @@ void NotificationServer::invokeAction(unsigned int id, const QString &action) {
     Q_EMIT ActionInvoked(id, action);
 }
 
-
-#define INTERACTIVE_HINT "x-canonical-switch-to-application"
-
-
-
-
 QStringList NotificationServer::GetCapabilities() const {
     QStringList capabilities;
     capabilities.push_back("actions");
@@ -51,16 +46,23 @@ QStringList NotificationServer::GetCapabilities() const {
     capabilities.push_back("body-markup");
     capabilities.push_back("icon-static");
     capabilities.push_back("image/svg+xml");
+    capabilities.push_back(VALUE_HINT);
+    capabilities.push_back(VALUE_TINT_HINT);
     capabilities.push_back(URGENCY_HINT);
+    capabilities.push_back(SOUND_HINT);
+    capabilities.push_back(SUPPRESS_SOUND_HINT);
     capabilities.push_back(SYNCH_HINT);
-    capabilities.push_back(APPEND_HINT);
     capabilities.push_back(ICON_ONLY_HINT);
-    capabilities.push_back(BUTTON_TINT_HINT);
+    capabilities.push_back(AFFIRMATIVE_TINT_HINT);
+    capabilities.push_back(REJECTION_TINT_HINT);
     capabilities.push_back(TRUNCATION_HINT);
     capabilities.push_back(SNAP_HINT);
     capabilities.push_back(SECONDARY_ICON_HINT);
+    capabilities.push_back(NON_SHAPED_ICON_HINT);
     capabilities.push_back(MENU_MODEL_HINT);
     capabilities.push_back(INTERACTIVE_HINT);
+    capabilities.push_back(TIMEOUT_HINT);
+    capabilities.push_back(SWIPE_HINT);
 
     return capabilities;
 }
@@ -82,7 +84,13 @@ Notification* NotificationServer::buildNotification(NotificationID id, const Hin
         expireTimeout = 3000;
         ntype = Notification::Type::Confirmation;
     } else if (hints.find(SNAP_HINT) != hints.end()) {
-        expireTimeout = 60000;
+        QVariant u = hints[TIMEOUT_HINT].variant();
+        if(!u.canConvert(QVariant::Int)) {
+            expireTimeout = 60000;
+        } else {
+            expireTimeout = u.toInt();
+        }
+
         ntype = Notification::Type::SnapDecision;
     } else if(hints.find(INTERACTIVE_HINT) != hints.end()) {
         ntype = Notification::Type::Interactive;
@@ -101,7 +109,7 @@ unsigned int NotificationServer::Notify (const QString &app_name, unsigned int r
         const QStringList &actions, const Hints &hints, int expire_timeout) {
     const unsigned int FAILURE = 0; // Is this correct?
     const int minActions = 4;
-    const int maxActions = 12;
+    const int maxActions = 14;
     //QImage icon(app_icon);
     int currentId = idCounter;
     QSharedPointer<Notification> notification;
@@ -112,33 +120,7 @@ unsigned int NotificationServer::Notify (const QString &app_name, unsigned int r
         }
         currentId = replaces_id;
         notification = model.getNotification(replaces_id);
-        // Appending text is a special case. This is the new update-behaviour
-        // for the append-hint and expects the client-app to use libnotify's
-        // call notify_notification_update()
-        if (hints.find(APPEND_HINT) != hints.end()) {
-            QString newBody = QString(notification->getBody() + "\n" + body);
-            notification->setBody(newBody);
-            model.notificationUpdated(currentId);
-            return notification->getID();
-        }
-        // Otherwise we let the code below update the fields.
     } else {
-
-        // support append-hint also for a new notification using just the
-        // summary-text as "identifier"... this is legacy behaviour and
-        // only kept for compatibility-reasons... developer-docs have to
-        // be updated and extended with a remark that this will be dropped
-        // by the next cycle
-        if (hints.find(APPEND_HINT) != hints.end()) {
-            notification = model.getNotification(summary);
-            if (notification) {
-                QString newBody = QString(notification->getBody() + "\n" + body);
-                notification->setBody(newBody);
-                model.notificationUpdated(notification->getID());
-                return notification->getID();
-            }
-        }
-
         Notification *n = buildNotification(currentId, hints);
         if(!n) {
             fprintf(stderr, "Could not build notification object.\n");
@@ -150,6 +132,7 @@ unsigned int NotificationServer::Notify (const QString &app_name, unsigned int r
             idCounter = 1;
     }
 
+    Q_ASSERT(notification);
     if(notification->getType() == Notification::Type::Interactive) {
         int numActions = actions.size();
         if(numActions != 2) {
@@ -193,7 +176,12 @@ unsigned int NotificationServer::Notify (const QString &app_name, unsigned int r
     QVariant secondaryIcon = hints[SECONDARY_ICON_HINT].variant();
     notification->setSecondaryIcon(secondaryIcon.toString());
 
-    if(replaces_id == 0) {
+    QVariant value = hints[VALUE_HINT].variant();
+    notification->setValue(value.toInt());
+
+    if(replaces_id) {
+        model.notificationUpdated(currentId);
+    } else {
         model.insertNotification(notification);
     }
     return currentId;
